﻿Public Class DownloadSeqs
    ' This delegate enables asynchronous calls for setting
    ' the text property on a TextBox control.
    Delegate Sub SetTextCallback(ByVal [text] As String)

    Dim g_NumSeqs As Integer
    Dim g_awSeqStart(20) As UInt16
    Dim g_awSeqSizes(20) As UInt16
    Dim g_abMem(32768) As Byte
    Dim g_iSelSeq As Integer
    Public g_sCommPort As String
    Public g_sUserInput As String
    Public g_fSendString As Boolean

    Public XBSW As Stopwatch

    Private Sub DownloadSeqs_FormClosed(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed
        My.Settings.MainFormSize = Size
        My.Settings.Splitter1 = SplitContainer1.SplitterDistance
        My.Settings.Splitter2 = SplitContainer2.SplitterDistance
    End Sub




    Private Sub DownloadSeqs_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' Show all available COM ports.
        Dim rk As RegistryKey = Registry.CurrentUser
        Dim rkDLS As RegistryKey = rk.CreateSubKey("software\DownloadSeqs")
        Dim x As Object
        g_iSelSeq = -1 ' make sure we will display the right one...

        ' try to restore the last filename so we don't have to do a browse every time
        x = rkDLS.GetValue("FileName")
        If x <> Nothing Then
            FileName.Text = x.ToString()
        End If

        For Each sp As String In My.Computer.Ports.SerialPortNames
            ComLB.Items.Add(sp)
        Next

        Size = My.Settings.MainFormSize   ' see if we can save and restore the size

        If My.Settings.Splitter2 Then
            SplitContainer1.SplitterDistance = My.Settings.Splitter1
            SplitContainer2.SplitterDistance = My.Settings.Splitter2
        End If



        If (ComLB.Items.Count <> 0) Then

            x = rkDLS.GetValue("CommPort")
            If x <> Nothing Then
                ComLB.SelectedIndex = ComLB.FindString(x.ToString)

            Else
                ComLB.SelectedIndex = ComLB.Items.Count - 1
            End If
        End If

        If ComLB.SelectedIndex <> -1 Then
            Try

                'bugbug: try to communicate with Bap so we can reset it now instead of later
                Using com1 As IO.Ports.SerialPort = _
                            My.Computer.Ports.OpenSerialPort(ComLB.Items(ComLB.SelectedIndex), 38400, IO.Ports.Parity.None)
                    com1.Handshake = IO.Ports.Handshake.None
                    com1.DtrEnable = False
                End Using
            Catch ex As Exception

            End Try

        End If

        ' create our timer
        XBSW = New Stopwatch()
        g_fSendString = False


    End Sub

    Private Sub ComLB_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ComLB.SelectedIndexChanged
        ' Show all available COM ports.
        Dim rk As RegistryKey = Registry.CurrentUser
        Dim rkDLS As RegistryKey = rk.CreateSubKey("software\DownloadSeqs")

        If (ComLB.SelectedIndex >= 0) Then
            rkDLS.SetValue("CommPort", ComLB.Items(ComLB.SelectedIndex).ToString)
            Try

                Using com1 As IO.Ports.SerialPort = _
                My.Computer.Ports.OpenSerialPort(ComLB.Items(ComLB.SelectedIndex), 38400, IO.Ports.Parity.None)
                    com1.Handshake = IO.Ports.Handshake.None
                    com1.DtrEnable = False

                End Using
            Catch ex As Exception

            End Try

        Else
            ' nothing selected, disable a few controls.
        End If
        rkDLS.Close()
        rk.Close()
    End Sub

    Private Sub FileName_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FileName.TextChanged
        Dim rk As RegistryKey = Registry.CurrentUser
        Dim rkDLS As RegistryKey = rk.CreateSubKey("software\DownloadSeqs")

        If (ComLB.SelectedIndex >= 0) Then
            rkDLS.SetValue("FileName", FileName.Text)
        Else
            ' nothing selected, disable a few controls.
        End If
        rkDLS.Close()
        rk.Close()

    End Sub

    Private Sub Browse_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Browse.Click
        OpenFileDialog1.Filter = "EEProm File (*.eep)|*.EEP" & _
            "|PEP CSV files (*.csv)|*.CSV" & _
            "|All Files (*.*)|*.*"
        OpenFileDialog1.FilterIndex = 1
        OpenFileDialog1.DefaultExt = "EEP"
        OpenFileDialog1.ShowDialog()
        FileName.Text = OpenFileDialog1.FileName()
    End Sub

    Private Sub Download_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Download.Click
        Dim abTemp(100) As Byte ' 100 bytes to read junk into
        Dim cbOutput As UInt16
        Dim ab(1) As Byte

        ' Now we have a listview of sequences with checkboxs, so lets use these checkboxs to decide which 
        ' of the sequences to download.  
        Dim i As Integer
        Dim cbUsed As UInt16 = 0                        ' The size of data to download will depend on checked items...
        Dim cSeqsToDownload As Integer = 0

        For i = 0 To LVSeqHeaders.Items.Count - 1
            If LVSeqHeaders.Items.Item(i).Checked Then
                cSeqsToDownload = cSeqsToDownload + 1   ' We have to to download!
                cbUsed = cbUsed + g_awSeqSizes(i)
            End If
        Next

        If cSeqsToDownload > 0 Then
            Try

                Using com1 As IO.Ports.SerialPort = _
                    My.Computer.Ports.OpenSerialPort(ComLB.Items(ComLB.SelectedIndex), 38400, IO.Ports.Parity.None)
                    com1.NewLine = Chr(13)
                    com1.Handshake = IO.Ports.Handshake.None
                    com1.BaseStream.Flush()

                    LBTerminal.Items.Add("Start Download")
                    Dim j As Integer
                    Dim wChksum As UInt16
                    Dim abPacket(6) As Byte
                    Dim wStart As UInt16
                    Dim wSeqStart As UInt16 = &H100 ' we will start the first sequence here...
                    Dim cbLeft As UInt16

                    ' For Version 2 convert to first line is an ascii command line in the form:
                    'S <cSeqsToDownload> <CB>'
                    Dim sStartPacket As String = "s " + cSeqsToDownload.ToString + " " + cbUsed.ToString + Chr(13)
                    com1.Write(sStartPacket)
                    'com1.WriteLine(sStartPacket)
                    cbOutput = sStartPacket.Length
                    wChksum = cSeqsToDownload + (cbUsed >> 8) + (cbUsed And &HFF)
                    abPacket(5) = wChksum And &HFF

                    com1.BaseStream.Flush()

                    System.Threading.Thread.Sleep(25) ' Give other side a little time to get back into read
                    If com1.BytesToRead >= cbOutput Then
                        com1.Read(abTemp, 0, cbOutput)
                    End If
                    com1.Read(ab, 0, 1)

                    'I am expecting a one Ack which is the negative of the checksum I wrote out.
                    If ab(0) <> Not abPacket(5) Then
                        Throw New System.Exception("Main Header Ack Error.")
                    End If
                    LBTerminal.Items.Add("Top Level Info downloaded")
                    System.Threading.Thread.Sleep(250) ' Give other side a little time to do any processing and get back to read...

                    cSeqsToDownload = 0         ' reuse for this loop as well
                    For i = 0 To g_NumSeqs - 1
                        ' We will pass a calculated Start location instead of the one from the file as we may not download all of the
                        ' sequences and we wish to keep the sequences one after another...
                        If LVSeqHeaders.Items.Item(i).Checked Then
                            wStart = wSeqStart                      ' setup the sequence to start at the next location
                            cbLeft = g_awSeqSizes(i)
                            abPacket(0) = cSeqsToDownload           ' Index by the count of actual ones downloaded
                            abPacket(1) = wSeqStart >> 8      ' Next 2 bytes is where we think it starts
                            abPacket(2) = wSeqStart And &HFF
                            abPacket(3) = cbLeft >> 8      ' Next 2 bytes is the size
                            abPacket(4) = cbLeft And &HFF

                            wStart = g_awSeqStart(i)
                            g_abMem(wStart) = cSeqsToDownload       ' The SSC-32 uses the seqnumber to confirm validity - so update it here...
                            wSeqStart = wSeqStart + g_awSeqSizes(i) ' Setup the starting address for the next one we download 
                            cSeqsToDownload = cSeqsToDownload + 1   ' update for the next one...
                            wChksum = 0
                            For j = 0 To 4 ' try to avoid overlflows
                                wChksum = wChksum + abPacket(j)
                            Next
                            abPacket(5) = wChksum And &HFF
                            ProgressBar.Minimum = 0
                            ProgressBar.Maximum = cbLeft + 32
                            ProgressBar.Step = 32
                            ProgressBar.Value = 0
                            LBTerminal.Items.Add("Seq " + LVSeqHeaders.Items.Item(i).SubItems.Item(0).Text + " Start")
                            com1.ReadTimeout = 250 'wait a maximum of .250 seconds for a response
                            System.Threading.Thread.Sleep(25) ' Give other side a little time to get back into read
                            com1.Write(abPacket, 0, 6)
                            com1.BaseStream.Flush()
                            cbOutput = 6

                            System.Threading.Thread.Sleep(25) ' Give other side time to maybe echo this back
                            If com1.BytesToRead >= cbOutput Then
                                com1.Read(abTemp, 0, cbOutput)
                            End If

                            com1.Read(ab, 0, 1)
                            If ab(0) <> Not abPacket(5) Then
                                Throw New System.Exception("Seq Header Ack Error.")
                            End If
                            ProgressBar.PerformStep()
                            LBTerminal.Items.Add("Seq " + LVSeqHeaders.Items.Item(i).SubItems.Item(0).Text + " 32 byte buffs")
                            While cbLeft >= 32
                                wChksum = 0
                                For j = 0 To 31
                                    wChksum = wChksum + g_abMem(wStart + j)
                                Next
                                abPacket(5) = wChksum And &HFF
                                System.Threading.Thread.Sleep(10) ' Give other side a little time to get back into read
                                com1.Write(g_abMem, wStart, 32) ' write out the next 32 bytes
                                com1.Write(abPacket, 5, 1)  ' write out the checksum
                                cbOutput = 33
                                com1.BaseStream.Flush()
                                System.Threading.Thread.Sleep(10) ' Give other side a little time to echo back...
                                If (com1.BytesToRead > 1) And (com1.BytesToRead < cbOutput) Then
                                    System.Threading.Thread.Sleep(25) ' Give other side a little time to get back into read
                                End If
                                If com1.BytesToRead >= cbOutput Then
                                    com1.Read(abTemp, 0, cbOutput)
                                End If
                                com1.Read(ab, 0, 1)
                                If ab(0) <> Not abPacket(5) Then
                                    Throw New System.Exception("Data Ack Error.(" + wStart.ToString + " " + ab(0).ToString + _
                                                               " " + abPacket(5).ToString + ")")
                                End If
                                wStart = wStart + 32
                                cbLeft = cbLeft - 32
                                ProgressBar.PerformStep()
                                Application.DoEvents()          ' Give the UI a chance to update
                            End While

                            If cbLeft Then
                                wChksum = 0
                                For j = 0 To cbLeft - 1
                                    wChksum = wChksum + g_abMem(wStart + j)
                                Next
                                abPacket(5) = wChksum And &HFF
                                System.Threading.Thread.Sleep(10) ' Give other side a little time to get back into read
                                com1.Write(g_abMem, wStart, cbLeft) ' write out the last bytes
                                com1.Write(abPacket, 5, 1)  ' write out the checksum
                                cbOutput = cbLeft + 1
                                com1.BaseStream.Flush()
                                LBTerminal.Items.Add("Seq " + LVSeqHeaders.Items.Item(i).SubItems.Item(0).Text + " Last buff " + _
                                                     cbLeft.ToString + " Bytes")

                                System.Threading.Thread.Sleep(10) ' Give other side a little time to get back into read
                                If (com1.BytesToRead > 1) And (com1.BytesToRead < cbOutput) Then
                                    System.Threading.Thread.Sleep(25) ' Give other side a little time to get back into read
                                End If
                                If com1.BytesToRead >= cbOutput Then
                                    com1.Read(abTemp, 0, cbOutput)
                                End If
                                com1.Read(ab, 0, 1)
                                If ab(0) <> Not abPacket(5) Then
                                    Throw New System.Exception("Data Ack Error-last.")
                                End If
                            End If
                            ProgressBar.PerformStep()
                            LBTerminal.Items.Add("Seq " + LVSeqHeaders.Items.Item(i).SubItems.Item(0).Text + " downloaded - Wait Final Ack")
                            Application.DoEvents()          ' Give the UI a chance to update

                            ' At end of a sequence give other side a chance to do their work.  Wait up to 5 seconds for another ack.
                            com1.ReadTimeout = 5000 'wait a maximum of 5 seconds for a response
                            com1.Read(ab, 0, 1)
                            If ab(0) <> Not abPacket(5) Then
                                Throw New System.Exception("Write Sequence Ack Error.")
                            End If
                            LBTerminal.TopIndex = LBTerminal.Items.Add("Seq " + LVSeqHeaders.Items.Item(i).SubItems.Item(0).Text + " downloaded - Completed") - 1
                            Application.DoEvents()          ' Give the UI a chance to update
                            System.Threading.Thread.Sleep(25) ' Give other side time to get back into read.
                        End If

                    Next


                End Using
            Catch ex As Exception
                LBTerminal.Items.Add("Error in Download:" + ex.Message)
            End Try
        Else
            LBTerminal.Items.Add("No items are Checked")
        End If
        If LBTerminal.Items.Count > 0 Then
            LBTerminal.TopIndex = LBTerminal.Items.Count - 1
        End If
        ProgressBar.Value = 0
    End Sub

    Private Sub ReadCSVFile(ByRef fileCSV As System.IO.StreamReader)
        Dim stItem As String
        Dim stFields() As String

        Dim aiPinNums(32) As Byte       ' build an array of pin numbers.  Assume max of 32 bins from Pep...
        Dim cPinNums As Byte         ' count of items in the array...

        Dim i As Integer
        Dim iSeqPrev As Byte = &HFF ' should not match the first line
        Dim cSeqs As Byte = 0
        Dim cSteps As Byte
        Dim iSeqNum As Byte
        Dim iMemAdr As UInt16 = &H100   ' next memory location we will write to
        Dim iMemHdr As UInt16           ' pointer to the header of the last item we processed.
        Dim wStepTime As UInt16
        Dim PinAngle As Single
        Dim wPulseWidth As UInt16       ' converted pulse width
        ' First line will be important to figure out what pins are defined and the like.
        Debug.Print("Read and Parse title line from CSV file")
        stItem = fileCSV.ReadLine()
        stFields = Split(stItem, ";")
        ' Guess that the count of pin numbers is equal to how many fields we got - 6 / 2...
        '     Project sequence Step Pinx pin... Tx t... TMAX TTOTAL INFO1
        If stFields.Count < 18 Then
            Throw New System.Exception("CSV file format error")
        End If
        cPinNums = (stFields.Count - 6) / 2
        For i = 0 To cPinNums - 1
            ' could make the testing more robust here...
            aiPinNums(i) = Integer.Parse((Mid(stFields(i + 3), 4)), Globalization.NumberStyles.Integer)
            Debug.Print("Pin : " + aiPinNums(i).ToString)
        Next

        ' Ok now lets loop through all of the items and see how well we can convert to the EEP format...
        While Not fileCSV.EndOfStream
            stItem = fileCSV.ReadLine()
            stFields = Split(stItem, ";")
            iSeqNum = Integer.Parse(stFields(1), Globalization.NumberStyles.Integer)
            If iSeqNum <> iSeqPrev Then
                ' if we were previously processing a sequence update the header information with the count of steps in sequence
                If cSeqs <> 0 Then
                    g_abMem(iMemHdr + 2) = cSteps                   ' Ok update the count of steps in the previous sequence
                    g_abMem(iMemHdr + cPinNums * 3 + 3) = wStepTime >> 8      ' save away the time if the last step...
                    g_abMem(iMemHdr + cPinNums * 3 + 4) = wStepTime And &HFF
                End If
                ' we need to start up a new sequence.
                ' need to write pointer at start of memory to point where we are stuffing this stuff...
                g_abMem(cSeqs * 2) = iMemAdr >> 8
                g_abMem(cSeqs * 2 + 1) = iMemAdr And &HFF
                cSeqs = cSeqs + 1       ' say we have a new sequence.

                ' now lets startup writing out the start of the sequence
                iMemHdr = iMemAdr   ' remember where we start this sequence
                g_abMem(iMemHdr) = iSeqNum
                g_abMem(iMemHdr + 1) = cPinNums
                iMemAdr = iMemAdr + 3   ' point to where pin number stuff is saved
                For i = 0 To cPinNums - 1
                    g_abMem(iMemAdr) = aiPinNums(i)
                    g_abMem(iMemAdr + 1) = &HB   ' set pin range to 3000
                    g_abMem(iMemAdr + 2) = &HB8
                    iMemAdr = iMemAdr + 3
                Next
                iMemAdr = iMemAdr + 2 ' skip the Time for step N-1
                iSeqPrev = iSeqNum
                cSteps = 0
            End If

            ' now lets process the step.
            For i = 0 To cPinNums - 1
                PinAngle = Single.Parse(Replace(stFields(i + 3), ",", "."))
                ' bugbug: may add new panel to compute the conversion factor.  This one is based on how we conver
                ' angles in the BAP for the movement function in the brat... That is Angle *166.6 = Hservo (bap28)
                ' to convert HSERVO to pulse: hservo = (msPulse * 16) - 24000 so 
                ' pinAngle * 166.6 = wPulseWidth * 16 - 24000
                ' wPulswidth = (pinAngle * 166.6) + 24000)/16 or 
                ' wPulseWidth = 1500 + pinAngle * 166.6/16
                'wPulseWidth = 1500 + (PinAngle / 90.0) * 1000.0       ' Note we should maybe parameterize this conversion...
                wPulseWidth = 1500.0 + (PinAngle * 166.6) / 16.0       ' Note we should maybe parameterize this conversion...
                g_abMem(iMemAdr) = wPulseWidth >> 8
                g_abMem(iMemAdr + 1) = wPulseWidth And &HFF
                iMemAdr = iMemAdr + 2       ' setup for the next pin
            Next
            'Now extract the time for the time for this setop
            wStepTime = Integer.Parse(stFields(stFields.Count - 2))
            g_abMem(iMemAdr) = wStepTime >> 8
            g_abMem(iMemAdr + 1) = wStepTime And &HFF
            iMemAdr = iMemAdr + 2       ' setup for the next step
            cSteps = cSteps + 1
        End While
        ' if we have an open sequence going we need to still update the count of steps and the time field
        If cSeqs <> 0 Then
            g_abMem(iMemHdr + 2) = cSteps                   ' Ok update the count of steps in the previous sequence
            g_abMem(iMemHdr + cPinNums * 3 + 3) = wStepTime >> 8      ' save away the time if the last step...
            g_abMem(iMemHdr + cPinNums * 3 + 4) = wStepTime And &HFF
        End If
        ' Mark where the next sequence would start as empty...
        g_abMem(cSeqs * 2) = 0
        g_abMem(cSeqs * 2 + 1) = 0

    End Sub

    Private Sub ReadEEPFile(ByRef fileEEP As System.IO.StreamReader)
        Dim stItem As String
        Dim i As Integer
        Dim iAddr As UInt32
        Dim stFields() As String
        stItem = fileEEP.ReadLine() ' First ignore the line with the comments on it...
        Debug.Print("First load file into memory")

        While Not fileEEP.EndOfStream
            stItem = fileEEP.ReadLine()
            stFields = Split(stItem, ";")
            iAddr = Integer.Parse(stFields(0), Globalization.NumberStyles.HexNumber)
            ' This EEP file is a hex dump with 15 bytes per line
            If iAddr < 32768 Then ' Actually we could probably bail once we get here...
                For i = 0 To 15
                    g_abMem(iAddr + i) = Integer.Parse(stFields(i + 1), Globalization.NumberStyles.HexNumber)
                Next
            End If
            ' System.Threading.Thread.Sleep(50) ' go slow so other side has a chance to process.
        End While

    End Sub

    Private Sub ReadSeqs_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ReadSeqs.Click
        Dim fileEEP As System.IO.StreamReader
        Dim list_item As ListViewItem

        LVSeqHeaders.BeginUpdate()
        LVSeqHeaders.Clear()
        LVSeqHeaders.Columns.Add("Seq #")
        LVSeqHeaders.Columns.Add("Servos")
        LVSeqHeaders.Columns.Add("Steps")
        LVSeqHeaders.Columns.Add("Size")
        g_iSelSeq = -1 ' make sure we will display the right one...

        Dim stFileName As String
        stFileName = FileName.Text
        Try

            Dim stExt = IO.Path.GetExtension(stFileName)
            Debug.Print("File Extension: " + stExt)
            fileEEP = My.Computer.FileSystem.OpenTextFileReader(stFileName)

            ' Figure out which read function to use to load the sequences into memory...
            If String.Compare(stExt, ".CSV", StringComparison.CurrentCultureIgnoreCase) = 0 Then
                ReadCSVFile(fileEEP)
            Else
                ReadEEPFile(fileEEP)
            End If
            fileEEP.Close()
            Debug.Print("First lets try seeing how many sequences")
            g_NumSeqs = 0
            While (g_abMem(g_NumSeqs * 2) <> 0) Or (g_abMem(g_NumSeqs * 2 + 1) <> 0)
                g_awSeqStart(g_NumSeqs) = g_abMem(g_NumSeqs * 2) * 256 + g_abMem(g_NumSeqs * 2 + 1)
                g_NumSeqs = g_NumSeqs + 1
            End While

            LBTerminal.Items.Add("Count of Seq: " + g_NumSeqs.ToString())

            For i = 0 To g_NumSeqs - 1
                ' First lets try to read until we find the beginning of the next sequence.
                ' We are assuming that the sequences follow each other...

                Dim iSeqNum As Byte = g_abMem(g_awSeqStart(i))
                Dim cServos As Byte = g_abMem(g_awSeqStart(i) + 1)
                Dim cSTeps As Integer = g_abMem(g_awSeqStart(i) + 2)
                g_awSeqSizes(i) = 3 + (3 * cServos) + (2 * (cServos + 1) * cSTeps) + 2
                LBTerminal.Items.Add("S: " + iSeqNum.ToString() + " Start: " + g_awSeqStart(i).ToString +
                             " Servos: " + cServos.ToString() + _
                             " Steps:" + cSTeps.ToString() + " Total bytes: " + g_awSeqSizes(i).ToString())

                list_item = New ListViewItem
                list_item.SubItems(0).Text = iSeqNum.ToString()
                list_item.SubItems.Add(cServos.ToString())
                list_item.SubItems.Add(cSTeps.ToString)
                list_item.SubItems.Add(g_awSeqSizes(i).ToString())
                list_item.Checked = True
                LVSeqHeaders.Items.Add(list_item)
            Next
            If LVSeqHeaders.Items.Count > 0 Then
                LVSeqHeaders.FocusedItem = LVSeqHeaders.Items(0)
                LVSeqHeaders.Items.Item(0).Selected = True
            End If
            Download.Enabled = True
            LVSeqHeaders.AutoResizeColumns(ColumnHeaderAutoResizeStyle.HeaderSize)

            ' lets see if we can make sure or status listbox shows our last command
            If LBTerminal.Items.Count > 0 Then
                LBTerminal.TopIndex = LBTerminal.Items.Count - 1
            End If
        Catch ex As Exception
            LBTerminal.Items.Add("Error in Loading Sequences:" + ex.Message)
        End Try
        LVSeqHeaders.EndUpdate()
    End Sub

    Private Sub LVSeqHeaders_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles LVSeqHeaders.SelectedIndexChanged
        Try
            'lets try to display information about the selected sequence
            Dim iSel As Int16 = LVSeqHeaders.FocusedItem.Index
            If (iSel <> -1) And (iSel <> g_iSelSeq) Then
                Dim iSeqNum As Byte = g_abMem(g_awSeqStart(iSel))
                Dim cServos As Byte = g_abMem(g_awSeqStart(iSel) + 1)
                Dim cSTeps As Integer = g_abMem(g_awSeqStart(iSel) + 2)

                LVSeqDetails.BeginUpdate()     ' let the system know we have just begun our update
                LVSeqDetails.Clear()
                LVSeqDetails.Scrollable = True

                g_iSelSeq = iSel

                LVSeqDetails.Columns.Add("Seq")
                LVSeqDetails.Columns.Add("Time")

                ' First Walk through the Servo/Speed list.  We will only extract the servos.
                Dim iMem As UInt16 = g_awSeqStart(iSel) + 3
                Dim iPin As Byte
                For iPin = 0 To cServos - 1
                    Dim bPin As Byte = g_abMem(iMem)
                    LVSeqDetails.Columns.Add(bPin.ToString)
                    iMem = iMem + 3
                Next
                ' Now lets walk through and show the steps
                iMem = iMem + 2 ' ignore the n-1 time
                Dim iStep As Byte
                Dim uT As UInt16
                Dim list_item As ListViewItem

                For iStep = 0 To cSTeps - 1
                    list_item = New ListViewItem
                    list_item.SubItems(0).Text = iStep.ToString
                    list_item.SubItems.Add("")
                    For iPin = 1 To cServos
                        uT = g_abMem(iMem)
                        uT = uT * 256 + g_abMem(iMem + 1)
                        iMem = iMem + 2
                        list_item.SubItems.Add(uT.ToString)
                    Next
                    ' now lets get the sequence step time
                    uT = g_abMem(iMem)
                    uT = uT * 256 + g_abMem(iMem + 1)
                    iMem = iMem + 2
                    list_item.SubItems(1).Text = uT.ToString
                    LVSeqDetails.Items.Add(list_item)
                Next
                LVSeqDetails.AutoResizeColumns(ColumnHeaderAutoResizeStyle.ColumnContent)
                LVSeqDetails.EndUpdate()
            End If
        Catch
        End Try


    End Sub

    Private Sub CBSeqList_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs)

    End Sub

    Private Sub CPBtn_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CPBtn.Click
        If ChangePins.ShowDialog() = DialogResult.OK Then
            Dim cChangepins As Integer = 0
            While (cChangepins < 32) And (ChangePins.g_abPinOrder(cChangepins) <> &HFF)
                cChangepins = cChangepins + 1
            End While
            Debug.Print("Change Pins " + cChangepins.ToString)


            If (g_NumSeqs > 0) And (cChangepins > 0) Then
                LBTerminal.Items.Add("Updating Pins in sequences")
                Dim iSel As Integer
                For iSel = 0 To g_NumSeqs - 1
                    Dim iSeqNum As Byte = g_abMem(g_awSeqStart(iSel))
                    Dim cServos As Byte = g_abMem(g_awSeqStart(iSel) + 1)
                    If cChangepins >= cServos Then

                        ' First Walk through the Servo/Speed list.  We will only extract the servos.
                        Dim iMem As UInt16 = g_awSeqStart(iSel) + 3
                        Dim iPin As Byte
                        For iPin = 0 To cServos - 1
                            g_abMem(iMem) = ChangePins.g_abPinOrder(iPin)
                            iMem = iMem + 3
                        Next
                    End If
                Next
                LBTerminal.Items.Add("Updating complete")
                g_iSelSeq = -1  ' clear what we think is the selected on...
                LVSeqHeaders.Items.Item(0).Selected = True
            End If

        End If
    End Sub

    Private Sub LBTerminal_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles LBTerminal.MouseDown
        If e.Button = Windows.Forms.MouseButtons.Right Then
            LBTerminal.ContextMenuStrip = PMTerminal
        End If

    End Sub

    Private Sub LBTerminalClear_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles LBTerminalClear.Click
        LBTerminal.Items.Clear() ' clear out the listbox
    End Sub

    Private Sub LBTerminalSelectAll_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles LBTerminalSelectAll.Click
        Dim i As Integer
        For i = 0 To LBTerminal.Items.Count - 1
            LBTerminal.SetSelected(i, True)
        Next

    End Sub

    Private Sub LBTerminalCopytoClipBoard_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles LBTerminalCopytoClipBoard.Click
        Dim i As Integer
        Dim s As String = ""
        For i = 0 To LBTerminal.Items.Count - 1
            If LBTerminal.GetSelected(0) Then
                s = s + LBTerminal.Items.Item(i).ToString + ControlChars.CrLf
            End If
        Next
        If s <> "" Then
            Clipboard.SetDataObject(s)
        End If

    End Sub

    Private Sub BTTerminal_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles BTTerminal.Click
        g_sCommPort = ComLB.Items(ComLB.SelectedIndex)
        If BTTerminal.Text = "Connect" Then
            Try

                BTTerminal.Text = "Disconnect"
                Download.Enabled = False    ' don't allow a download to happen while we are connected...
                ' Make sure that our packet data is setup properly
                CommThread.RunWorkerAsync()
            Catch ex As Exception
                Beep()
                Download.Enabled = True    ' Enable the download again

            End Try
        Else
            BTTerminal.Text = "Connect"
            CommThread.CancelAsync()            ' turn off the thread...
        End If

    End Sub

    Public Sub AddItemToLB(ByVal text As String)
        Dim iIndex As Integer = LBTerminal.Items.Add(text)
        LBTerminal.TopIndex = iIndex - 1

    End Sub


    '==============================================================================
    ' [AddLineToLBTerminal (S)]
    '
    ' This function will also switch to the appropriate display mode if necessary.
    '==============================================================================
    Public Sub AddLineToLBTerminal(ByRef S As String)
        ' Make sure we are in the right mode to display the data
        If LBTerminal.InvokeRequired Then
            Dim d As New SetTextCallback(AddressOf AddItemToLB)
            Me.Invoke(d, New Object() {S})
        Else
            LBTerminal.Items.Add(S)
        End If
        Return
    End Sub


    Private Sub CommThread_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles CommThread.DoWork
        Dim stLine As String
        CommThread.ReportProgress(0)
        XBSW.Start()

        Try
            Using com1 As IO.Ports.SerialPort = _
                My.Computer.Ports.OpenSerialPort(g_sCommPort, 38400, IO.Ports.Parity.None)
                com1.Handshake = IO.Ports.Handshake.None
                com1.DtrEnable = False
                com1.NewLine = Chr(13)

                com1.ReadTimeout = 200 'wait a maximum of .2 seconds for a complete ine

                While Not CommThread.CancellationPending              ' turn off the thread...
                    ' check to see if there is anything to display from the other side
                    If g_fSendString Then
                        com1.WriteLine(g_sUserInput)
                        g_fSendString = False
                    End If
                    If com1.BytesToRead > 0 Then
                        Try
                            stLine = com1.ReadLine()
                            AddLineToLBTerminal(stLine)
                        Catch
                            System.Threading.Thread.Sleep(10)  ' Sleep a bit so we are not just hung...
                        End Try
                    Else
                        'We got something we were not expecting so error out.	
                        System.Threading.Thread.Sleep(5)  ' Sleep a bit so we are not just hung...
                    End If
                End While
            End Using
        Catch
        End Try

        CommThread.ReportProgress(99)

    End Sub


    Private Sub CommThread_ProgressChanged(ByVal sender As Object, ByVal e As System.ComponentModel.ProgressChangedEventArgs) Handles CommThread.ProgressChanged
        If e.ProgressPercentage < 10 Then
            LBTerminal.TopIndex = LBTerminal.Items.Add("*** Thread Start ***") - 1
        ElseIf e.ProgressPercentage > 90 Then
            LBTerminal.TopIndex = LBTerminal.Items.Add("*** Thread Canceled ***") - 1
            ' Make sure that if the thread exited by something other than us, to reset button to show connect
            If BTTerminal.Text = "Disconnect" Then
                BTTerminal.Text = "Connect"
            End If
            Download.Enabled = True    ' Enable the download again

        End If
    End Sub

    Private Sub TBTerminal_KeyPress(ByVal sender As Object, ByVal e As System.Windows.Forms.KeyPressEventArgs) Handles TBTerminal.KeyPress
        If e.KeyChar = ControlChars.Cr Then
            g_sUserInput = TBTerminal.Text
            g_fSendString = True
            LBTerminal.Items.Add(">> " + g_sUserInput)
            LBTerminal.TopIndex = LBTerminal.Items.Count - 1
            TBTerminal.Text = ""
            e.Handled = True
        End If
    End Sub
End Class
