;==============================================================================
; SeqUtility.bas - a program that does several different things to help 
;		download sequences, view the sequences that are stored in the EEPROM
;		of an SSC-32, Dump EEProm memory...
;==============================================================================
; On an Arc32-Bap40 it assumes that sequences are stored internally on the Arc32
; On Bap28s it assumes that the sequences are stored on an SSC-32...
 

#ifdef TMB1		; this is defined for BAP40 and ARC-32
enablehserial
cSound			con P19	; lets try output sound on plug in speaker
#else
cSSC_OUT   		con P11		;Output pin for (SSC32 RX) on BotBoard (Yellow)
cSSC_IN    		con P10		;Input pin for (SSC32 TX) on BotBoard (Blue)
wSSC_BAUD		var word	;Will try to determine this
cSound			con 9
#endif

;==============================================================================
; Defines and Variables
;==============================================================================

; Used in download sequences
abHdr			var	byte(6)
wAddr			var	word
ab				var	byte(1900)	; save some room...
szSSCEEPromTemp	var	byte(12)	; use to output 
bChkSum			var byte
bChkSumIn		var byte
bAck			var	byte
cSeqs			var	byte
iSeqDl			var byte
cbTotal			var	word
i				var word

iSeq			var	byte
cbSeq			var	word
wSeqStart		var	word
cbLeft			var	word
wBufStart		var	word
fValid			var bit

swServo			var	sword

; Variables used in Dump
wDumpStart	var	word
wDumpCnt	var	word
iDump		var	byte
wB			var	word
sB			var sword

; Variables for Sequence Viewer
wGPSeqPtr		var word
bGPCntServos	var	byte			; how many servos are used in this sequence
bGPServoNum		var	byte
bGPCntSteps		var	byte			; how many moves are in this sequence...
bGPStepNum		var	byte			; which step are we on.
wGPStepTime		var	word			; How long this step will take

; Variables used in main command processing...
_TM_szIn		var	ab				; reuse main buffer byte(80)		; input prompt
_TM_iszIn		var	byte


_TM_bDumpMode	var	byte			; What mode are we in.  Simple stuff for now

_TM_wDumpStart	var	word			; what address to start at
_TM_wDumpCnt	var	word			; how many bytes to dump




;==============================================================================
; Complete initialization
;==============================================================================
	sound cSound,[50\3800, 50\4200, 40\4100]
	enable	; make sure interrupts are enabled
#ifdef TMB1
	SetHSerial H38400,H8DATABITS,HNOPARITY,H1STOPBITS
#else
	gosub FindSSCBaudRate		; put a function in to try to find out what the baud rate is...
	high p4
	high p5
	high p6
#endif	

	; setup initial values for where to start dumping from and count
	_TM_wDumpStart = 0
	_TM_wDumpCnt = 32		; dump 


;==============================================================================
; Main Loop
;==============================================================================
_TM_ShowPrompt:
#ifdef TMB1
	hserout [13, 13, |
#else
	serout s_out, i38400, [13, 13, |
#endif	
			"Sequence Download and Utilities",13,|
			"  D [<start addr>][<cnt>] - Dump EEPROM memory", 13, |
			"  S - Download sequence(s) - VB call only", 13, |
			"  V <n> - View Sequence", 13, |
			" : "]
Main:
#ifdef TMB1
	hserin [str _TM_szIn\80\13] 
#else
	serin s_in, i38400, [str _TM_szIn\80\13]
#endif		
	
	if _TM_szIn(0) = 13 then
		goto _TM_ShowPrompt

	elseif (_TM_szIn(0) = "d") or (_TM_szIn(0) = "D")	; The user wants to Display EEPROM memory
		gosub _TM_ProcessEEPromDumpCmd		
		
	elseif (_TM_szIn(0) = "s") or (_TM_szIn(0) = "S")	; The user wants to download a sequence or sequences...
		gosub _TM_ProcessDownloadSeqCmd
	elseif (_TM_szIn(0) = "v") or (_TM_szIn(0) = "V")	; The user wants to view a sequence.
		gosub _TM_ProcessViewSequenceCmd
	endif
#ifdef TMB1		
	hserout [": "]
#else
	serout s_out, i38400, [": "]
#endif	
goto Main:
;&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
; EEPROM - Dump code...
;==============================================================================
;0 - standard hex byte dump (h)
;1 - unsigned Decimal - words (w)
;2 - signed decimal words (s)
;------------------------------------------------------------------------------
_TM_ProcessEEPromDumpCmd:
	if _TM_szIn(1) = 13 then
		gosub DumpEEProm[_TM_wDumpStart, _TM_wDumpCnt]	
	elseif (_TM_szIn(1) = "h") or (_TM_szIn(1) = "H")
		_TM_bDumpMode = 0
	elseif (_TM_szIn(1) = "w") or (_TM_szIn(1) = "w")
		_TM_bDumpMode = 1
	elseif (_TM_szIn(1) = "s") or (_TM_szIn(1) = "S")
		_TM_bDumpMode = 2
	else
		_TM_iszIn = 1
		gosub ExtractNum[], _TM_wDumpStart
		if  _TM_szIn(_TM_iszIn) = "=" then
			; we are going to try to update the EEPROM!
			_TM_iszIn = _TM_iszIn + 1
			while _TM_szIn(_TM_iszIn) <> 13	; let walk through the items
				gosub ExtractNum[], _TM_wDumpCnt	; reuse this variable.
				; This pass of the code I will only update in bytes or words, but not signed
				if _TM_bDumpMode = 0 then
					Writedm _TM_wDumpStart,[_TM_wdumpCnt.lowbyte]
					_TM_wDumpStart = _TM_wDumpStart + 1
				else
					Writedm _TM_wDumpStart,[_TM_wdumpCnt.highbyte, _TM_wdumpCnt.lowbyte] ; Write 2 bytes
					_TM_wDumpStart = _TM_wDumpStart + 2
				endif
			wend	
		else
			if _TM_szIn(_TM_iszIn) = " " then
				; assume that we have a count specified here
				_TM_iszIn = _TM_iszIn + 1
				gosub ExtractNum[], _TM_wDumpCnt
			endif
			gosub DumpEEProm[_TM_wDumpStart, _TM_wDumpCnt]	
		endif
	endif		

	_TM_wDumpStart = _TM_wDumpStart + _TM_wDumpCnt ; assume we will start after the last write...
	return
	
							
;------------------------------------------------------------------------------
; Extract Number from input line
;------------------------------------------------------------------------------
wNum		var	word
fContinue	var	bit

ExtractNum:
	wNum = 0
	; skip any leading blanks...
	while (_TM_szIn(_TM_iszIn) = " ")
		_TM_iszIn = _TM_iszIn + 1
	wend

	; lets parse the strings to see if we have a start address or not...
	if (_TM_szIn(_TM_iszIn) = "0") and ((_TM_szIn(_TM_iszIn+1) = "x") or (_TM_szIn(_TM_iszIn+1) = "X")) then
		; hex number 
		_TM_iszIn = _TM_iszIn+2
		fContinue = 1
		do 
			if (_TM_szIn(_TM_iszIn) >= "0") and (_TM_szIn(_TM_iszIn) <= "9") then
				wNum = wNum * 16 + _TM_szIn(_TM_iszIn)-"0"
				_TM_iszIn = _TM_iszIn + 1
			elseif (_TM_szIn(_TM_iszIn) >= "a") and (_TM_szIn(_TM_iszIn) <= "f")
				wNum = wNum * 16 + _TM_szIn(_TM_iszIn)-"a"+10
				_TM_iszIn = _TM_iszIn + 1
			elseif (_TM_szIn(_TM_iszIn) >= "A") and (_TM_szIn(_TM_iszIn) <= "F")
				wNum = wNum * 16 + _TM_szIn(_TM_iszIn)-"A"+10
				_TM_iszIn = _TM_iszIn + 1
			else 
				fContinue = 0
			endif
		while(fcontinue)
	else
		; assume decimal number
		fContinue = 1
		do 
			if (_TM_szIn(_TM_iszIn) >= "0") and (_TM_szIn(_TM_iszIn) <= "9") then
				wNum = wNum * 10 + _TM_szIn(_TM_iszIn)-"0"
				_TM_iszIn = _TM_iszIn + 1
			else 
				fContinue = 0
			endif
		while(fcontinue)
	endif

	return wNum

;==============================================================================
; [_TM_ProcessDownloadSeqCmd]
;==============================================================================
_TM_ProcessDownloadSeqCmd:
	; The VB app will pass us an ascii string with the number of sequences and cbTotal as ascii.
	; need to extract that from the command line...
	_TM_iszIn = 1
	gosub ExtractNum[], cSeqs		; extract the count of sequences
	gosub ExtractNum[], cbTotal		; likewise the expected total number of bytes

	; do a little validation to make sure the user did not enter this by accident...
	; need at least one sequence
	if (cSeqs = 0) or (cSeqs > 20) or (cbTotal < (cSeqs*12)) or (cbTotal > 16383) then
		; either no steps or too many or not enough bytes for even 1 servo and step per sequence
		; or obviously won't fit in EEPROM, then probably garabage, so bail before we do anything
#ifdef TMB1
		hserout ["*** Download Command Error ***", 13]
#else		
		serout s_out, i38400, ["*** Download Command Error ***", 13]
#endif
		return
	endif			
	; Calculate return number that the other will understand as an ack
	; should add some more validation here
	bchksum = cSeqs + cbTotal.highbyte + cbTotal.lowbyte
	bAck = !bChksum
#ifdef TMB1
	hserout	[bAck]	' send an Ack back to the caller.
#else
	serout s_out, i38400,[bAck]	' send an Ack back to the caller.
#endif	
	gosub ClearSeqPtrs	; clear the sequence pointers

	; now we need to loop through all of the sequences that are supposed to be downloaded
	for iSeqDl = 0 to cSeqs -1
		; We should now get a header for the next sequnce
		; BUGBUG should probably put a timeout in here...
#ifdef TMB1		
		hserin [str abHdr\6]	' read in 5 byte header + 
#else
		serin s_in, i38400, [str abHdr\6]	' read in 5 byte header + 
#endif		
		bChkSum = abHdr(0)+abHdr(1)+abHdr(2)+abHdr(3)+abHdr(4)
	
		; we have hopefully a valid header...
		bAck = !bChksum		' invert what we received to send back		
#ifdef TMB1
		hserout	[bAck]	' send an Ack back to the caller.
#else
		serout s_out, i38400,[bAck]	' send an Ack back to the caller.
#endif	
		fValid = 1			; start off assuming the sequence is valid...
		iSeq = abHdr(0)

		wSeqStart.highbyte = abHdr(1)	; Where it was in EEPROM file. 
		wSeqStart.lowByte = abHdr(2)	
		cbSeq.highbyte = abHdr(3)		; This sequence size 
		cbSeq.lowByte = abHdr(4)	

		cbLeft = cbSeq
		wBufStart = 0
		while cbLeft >= 32
			; also put in timeout...
#ifdef TMB1
			hserin [str ab(wBufStart)\32, bChkSumIn]	' Read in next 32 bytes
#else
			toggle P5		; debug this stuff...
			serin s_in, i38400, [str ab(wBufStart)\32, bChkSumIn]	' Read in next 32 bytes
#endif
			bChkSum = 0

			for i = 0 to 31
				bChkSum = bchkSum + ab(wBufStart+i)
			next
			if bChksum <> bChksumIn then
				fValid = 0
			endif
			
			bAck = !bChksum		' invert what we received to send back		
#ifdef TMB1
			hserout	[bAck]	' send an Ack back to the caller.
#else
			serout s_out, i38400,[bAck]	' send an Ack back to the caller.
#endif	
			cbLeft = cbLeft - 32
			wBufStart = wBufStart + 32
		wend
		if cbLeft then
#ifdef TMB1
			hserin [str ab(wBufStart)\cbLeft, bChkSumIn]	' read in the last remaining bytes of this sequence
#else
			toggle P6		; debug this stuff...
			serin s_in, i38400,  [str ab(wBufStart)\cbLeft, bChkSumIn]	' read in the last remaining bytes of this sequence
#endif			
			bChkSum = 0
			for i = 0 to cbLeft-1
				bChkSum = bchkSum + ab(wBufStart+i)
			next
			if bChksum <> bChksumIn then
				fValid = 0
				toggle p4
			endif
			
			; we have hopefully a valid header...
			bAck = !bChksum		' invert what we received to send back		
#ifdef TMB1
			hserout	[bAck]	' send an Ack back to the caller.
#else
			serout s_out, i38400,[bAck]	' send an Ack back to the caller.
#endif	
		
		endif
		; After we finish receiving a sequence we can now process it.  At the end of that echo one last Ack to
		; let the other side know that we completed it...
		if fValid then
			gosub WriteSeqToSSC;				
		endif
#ifdef TMB1
		hserout	[bAck]	' Output the final Ack to the user to say we finished writing out this sequence
#else
		serout s_out, i38400, [bAck]	' final ack
#endif		

	next		; loop through all of the 
	return		; go back to the main input section...
	
; Handle a timeout 1 second should be plenty of time for VB app to send us data!!!
_TMPDL_TO:
#ifdef TMB1
	hserout ["*** Sequence Download Failed: Timeout ***", 13]
#else
	serout s_out, i38400, ["*** Sequence Download Failed: Timeout ***", 13]
#endif	
	return

;&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
;$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
;$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
; 				ARC 32 version
;$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
;$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
	
#ifdef TMB1
ARC32_SSC_OFFSET	con	0x400		; we will offset saving all of the SSC-32 by this amount
;===============================================================================
; WriteSeqToSSC
;===============================================================================
ClearSeqPtrs:	; clear the sequence pointers
	writedm ARC32_SSC_OFFSET,[rep 0\20]						; Clear 10 pointers.
	return
;------------------------------------------------------------------------------
; DumpEEProm:
;------------------------------------------------------------------------------
DumpEEProm[wDumpStart, wDumpCnt]:
	hserout ["Start Dump: ", hex wDumpStart, " Count: ", dec wDumpCnt, 13]
	while wDumpCnt > 0
		readdm	wDumpStart, [str ab\wDumpCnt max 16]		; read the bytes to dump max 16 at a time.
		
		hserout [hex4 wDumpStart\4,":"]
		
		; first dump the characters out in hex
		for iDump = 0 to 15
			if _TM_bDumpMode = 0 then
				if iDump < wDumpCnt then
					hserout [hex2 ab(iDump)\2, " "]
				else
					hserout ["   "]
				endif
			elseif ((iDump & 0x1) = 0)	; sort-of crap - but word size so only output every other byte
				if iDump < wDumpCnt then
					if _TM_bDumpMode = 1 then
						wB.highbyte = ab(iDump)
						wB.lowbyte = ab(iDump+1)
						hserout [dec5 wB\5, " "]
					else
						sB.highbyte = ab(iDump)
						sB.lowbyte = ab(iDump+1)
						if sB < 0 then
							hserout [sdec5 sB\5, " "]
						else
							hserout [sdec6 sB\6, " "]
						endif
					endif
				else
					hserout ["       "]
				endif
			endif
		next
		
		; then dump them out in ascii
		for iDump = 0 to ((wDumpCnt -1) max 15)
			if (ab(iDump) > 0x1f) and (ab(iDump) < 0x7f) then
				hserout [ab(iDump)]
			else
				hserout ["."]
			endif
		next
		
		hserout [13]	; output the cr to end the line
		; setup for the next line if any
		if wDumpCnt > 16 then
			wDumpCnt = wDumpCnt - 16
			wDumpStart = wDumpStart + 16
		else
			wDumpCnt = 0
		endif	
	wend	
	
	return	


;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@	
; View Sequences command processing
; Warning I am going to reuse the variables I used to run sequences.  WIll need to extract these
; if I wish to have a standalone version of this stuff
;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@	


_TM_ProcessViewSequenceCmd:
	if _TM_szIn(1) = 13 then
		_TM_wDumpStart = 0	; assume the first one.
	else
		_TM_iszIn = 1
		gosub ExtractNum[], _TM_wDumpStart
	endif
	
	; Now lets try to walk through the memory to dump out the data about the selected sequence.
	readdm ARC32_SSC_OFFSET + _TM_wDumpStart*2, [str wGPSeqPtr\2]
	; make sure there is a sequence...
	IF (wGPSeqPtr <> 0)  and (wGPSeqPtr <> 0xffff)	THEN
		; now lets get the header information, will skip sequence number
		readdm wGPSeqPtr+1,[bGPCntServos, bGPCntSteps]

		; now lets read in the pins
		wGPSeqPtr = wGPSeqPtr + 3 ; point to start of pin information.
		hserout ["Sequence: ", dec _TM_wDumpStart, " Cnt Servos: ", dec bGPCntServos, " Steps: ", dec bGPCntSteps, 13, |
				"Servos:        "]

		for bGPServoNum = 0 to bGPCntServos - 1
			readdm wGPSeqPtr, [ab]	; read in the pin, igore max value after
			hserout [dec4 ab\4, " "]
			wGPSeqPtr = wGPSeqPtr + 3
		next
		hserout [13]	; finish the servo line

		wGPSeqPtr = wGPSeqPtr + 2 ; skip over the N-1 time
		; now walk through the steps
		for bGPStepNum = 0 to bGPCntSteps-1 ;
			readdm wGPSeqPtr, [str ab\2*bGPCntServos, wGPStepTime.highbyte, wGPStepTime.lowByte] ; note did not change time bytes order ???
			hserout ["S: ", dec3 bGPStepNum\3, " ", dec4 wGPStepTime\4, " "]

			for bGPServoNum = 0 to bGPCntServos - 1
				; May want to convert back to Servo pulses as that is what the user is probably expecting...
				sB.highbyte = ab(bGPServoNum*2)
				sB.lowbyte = ab(bGPServoNum*2+1)
				hserout [dec4 (sB+30000)/20 \4, " "]
			next
		 	hserout [13]		
			wGPSeqPtr = wGPSeqPtr + 2*bGPCntServos + 2	; increment to the next sequence
		next
	endif

	return
						
;===============================================================================
; WriteSeqToSSC
;===============================================================================

WriteSeqToSSC:
	; first write out pointer to the data for this sequence.  We will offset the 
	; address by ARC32_SSC_OFFSET to save room for other stuff 
	wSeqStart = wSeqStart + ARC32_SSC_OFFSET

	; We will walk through the Sequence and convert the SSC type timings center 1500 to
	; Hservo values.  Wish we had some setting to verify that running at 20mhz, but since
	; this is only Bap40 or Arc32, must be 20mhz.
	; Calculate where the sequence information starts skip header and servo info
	wBufStart = 3 + ab(1) * 3 + 2;  Header size + 3 bytes per servo + Step n-1 time
	
	for i = 1 to ab(2)						; walk through each of the steps
		for cbLeft = 1 to ab(1)				; for each step walk through the PW for the servo
			swServo.highbyte = ab(wBufStart)
			swServo.lowbyte = ab(wBufStart + 1)	; Used signed variable should not be problem 3000 is about max
			
			;Convert to HSERVO value
			swServo = (swServo*20) - 30000	; should be HSERVO value for Arc32
			ab(wBufStart) = swServo.highbyte	; atom numbers are in different order
			ab(wBufStart+1) = swServo.lowbyte	
			wBufStart = wBufStart + 2		; get to the next one
		next	
		wBufStart = wBufStart + 2			; skip the time
	next			

	; first output the address to the start the of SSC - Note since this is all internal
	; we may as well offset the value and save it in our own number ordering...
	writedm ARC32_SSC_OFFSET+ (iSeq*2), [str wSeqStart\2]	; saves in our number system

	wBufStart = 0
	cbLeft = cbSeq
	if (wSeqStart & 0x1f) <> 0 then
		i = (32 - (wSeqStart & 0x1f))	; calculate how many bytes we are outputing
		writedm wSeqStart,[str ab(wBufStart)\i]
		cbLeft = cbLeft - i			; how many bytes did we output
		wBufStart = wBufStart + i
		wSeqStart = wSeqStart + i				; setup to the start of the next sequence
	endif

	while cbLeft >= 32
		writedm wSeqStart,[str ab(wBufStart)\32]
		cbLeft = cbLeft - 32			; how many bytes did we output
		wBufStart = wBufStart + 32
		wSeqStart = wSeqStart + 32				; setup to the start of the next sequence
	wend
	if cbLeft then
		writedm wSeqStart,[str ab(wBufStart)\cbLeft]
	endif
	
	return

;$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
;$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
;		BAP - SSC32 version
;$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
;$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
#else	

; SSC-32 version of stuff
CLSQSSTR	bytetable "EEW -0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0",13	; clear all of the first 10 seq ptrs
EEWSTR		bytetable "EEW -"
EERSTR		bytetable "EER -"
CRSTR		bytetable	13


;===============================================================================
; ReadSSCEEPromMem
;===============================================================================
_wSSCRStart		var	word	; start address to read
_cbSSCR			var byte	; count of bytes to read
_pabSSCR		var pointer	; pointer to buffer to read into
_cbSSCRRead		var	byte
ReadSSCEEPPromMem[_wSSCRStart, _cbSSCR, _pabSSCR]:
	gosub TASerOut[cSSC_OUT, wSSC_BAUD, @EERSTR, 5]		; output command to the SSC
	szSSCEEPromTemp = dec5 _wSSCRStart\5
	szSSCEEPromTemp(5)=";"
	szSSCEEPromTemp(6)= dec2 _cbSSCR\2
	szSSCEEPromTemp(8) = 13
	gosub TASerOut[cSSC_OUT, wSSC_BAUD, @szSSCEEPromTemp, 9]		; Output the read request to the SSC
	gosub TASerIn[cSSC_IN, 0, wSSC_BAUD, _pabSSCR, _cbSSCR, 100000, -1], _cbSSCRRead
	;serout s_out, i38400, ["DBG SSC Cmd: (", dec _cbSSCRRead, "):", str EERSTR\5, str szSSCEEPromTemp\10\13, 13]
	return

;===============================================================================
; View Sequences command processing -- SSC version
; Warning I am going to reuse the variables I used to run sequences.  WIll need to extract these
; if I wish to have a standalone version of this stuff
;===============================================================================
_TM_ProcessViewSequenceCmd:
	if _TM_szIn(1) = 13 then
		_TM_wDumpStart = 0	; assume the first one.
	else
		_TM_iszIn = 1
		gosub ExtractNum[], _TM_wDumpStart
	endif
	
	; Now lets try to walk through the memory to dump out the data about the selected sequence.
	gosub ReadSSCEEPPromMem[_TM_wDumpStart*2, 2,   @wGPSeqPtr]

	; make sure there is a sequence...
	IF (wGPSeqPtr <> 0)  and (wGPSeqPtr <> 0xffff)	THEN
		; now lets get the header information, will skip sequence number
		; I will preuse the Pins array to read into
		gosub ReadSSCEEPPromMem[wGPSeqPtr+1, 2,   @ab]
		bGPCntServos = ab(0)
		bGPCntSteps = ab(1)

		; now lets read in the pins
		wGPSeqPtr = wGPSeqPtr + 3 ; point to start of pin information.
		serout s_out, i38400,  ["Sequence: ", dec _TM_wDumpStart, " Cnt Servos: ", dec bGPCntServos, " Steps: ", dec bGPCntSteps, 13, |
				"Servos:        "]

		for bGPServoNum = 0 to bGPCntServos - 1
			gosub ReadSSCEEPPromMem[wGPSeqPtr, 1,   @ab]
			serout s_out, i38400,  [dec4 ab\4, " "]
			wGPSeqPtr = wGPSeqPtr + 3
		next
		serout s_out, i38400,  [13]	; finish the servo line

		wGPSeqPtr = wGPSeqPtr + 2 ; skip over the N-1 time
		; now walk through the steps
		for bGPStepNum = 0 to bGPCntSteps-1 ;
			if bGPCntServos > 16 then
				ab32	var	ab(32)
				gosub ReadSSCEEPPromMem[wGPSeqPtr, 32,  @ab] 			; Read in the first 16 servos
				gosub ReadSSCEEPPromMem[wGPSeqPtr+32, 2*bGPCntServos-32,  @ab32] ; reuse ab buffer to save mem...
			else
				' Can do all in one read...
				gosub ReadSSCEEPPromMem[wGPSeqPtr, 2*bGPCntServos,  @ab] ; reuse ab buffer to save mem...
			endif
			gosub ReadSSCEEPPromMem[wGPSeqPtr+2*bGPCntServos, 2,  @wGPStepTime]
			serout s_out, i38400,  ["S: ", dec3 bGPStepNum\3, " ", dec4 wGPStepTime\4, " "]

			for bGPServoNum = 0 to bGPCntServos - 1
				; May want to convert back to Servo pulses as that is what the user is probably expecting...
				wB.highbyte = ab(bGPServoNum*2)
				wB.lowbyte = ab(bGPServoNum*2+1)
				serout s_out, i38400,  [dec4 wb\4, " "]
			next
		 	serout s_out, i38400,  [13]		
			wGPSeqPtr = wGPSeqPtr + 2*bGPCntServos + 2	; increment to the next sequence
		next
	endif

	return
						

;===============================================================================
; WriteSeqToSSC
;===============================================================================
ClearSeqPtrs:	; clear the sequence pointers
	gosub TASerOut[cSSC_OUT, wSSC_BAUD, @CLSQSSTR, 47]		; output command to the SSC
	return

;===============================================================================
; WriteSeqToSSC
;===============================================================================
WriteSeqToSSC:
	; will need to write a max of 32 bytes at a time preferably starting at 32 bit boundry...
	; We will use the memory address that was passed us...
;	wSeqStart.highbyte = abHdr(1)	; Where it was in EEPROM file. 
;	cbSeq.highbyte = abHdr(3)		; This sequence size 

	; first output the address to the start the of SSC
	gosub TASerOut[cSSC_OUT, wSSC_BAUD, @EEWSTR, 5]		; output command to the SSC
	szSSCEEPromTemp = dec2 (iSeq * 2)\2		; address to write out start address of sequence
	szSSCEEPromTemp(2)=","
	szSSCEEPromTemp(3)= dec3 wSeqStart.highbyte\3
	szSSCEEPromTemp(6)=","
	szSSCEEPromTemp(7)= dec3 wSeqStart.lowbyte\3
	szSSCEEPromTemp(10) = 13
	gosub TASerOut[cSSC_OUT, wSSC_BAUD, @szSSCEEPromTemp, 11]		; save the start.
	pause 10
	toggle p6
	; now Lets see if we started off on a 32 bit boundary...
	wBufStart = 0
	cbLeft = cbSeq
	if (wSeqStart & 0x1f) <> 0 then
		; ok let write the first part out
		gosub TASerOut[cSSC_OUT, wSSC_BAUD, @EEWSTR, 5]		; output command to the SSC
		szSSCEEPromTemp = dec5 wSeqStart\5
		gosub TASerOut[cSSC_OUT, wSSC_BAUD, @szSSCEEPromTemp, 5]		; output command to the SSC
		szSSCEEPromTemp(0) = ","
		for i = 0 to 31-(wSeqStart & 0x1f)
			szSSCEEPromTemp(1)=dec3 ab(wBufStart)\3
			gosub TASerOut[cSSC_OUT, wSSC_BAUD, @szSSCEEPromTemp, 4]		; output next byte value
			wBufStart = wBufStart + 1
		next
		gosub TASerOut[cSSC_OUT, wSSC_BAUD, @CRSTR, 1]		; output the CR
		cbLeft = cbLeft - (32 - (wSeqStart & 0x1f))			; how many bytes did we output
		wSeqStart = (wSeqStart + 32) & 0xffe0				; setup to the start of the next sequence
		pause 10
		toggle p6
	endif
	while cbLeft >= 32
		gosub TASerOut[cSSC_OUT, wSSC_BAUD, @EEWSTR, 5]		; output command to the SSC
		szSSCEEPromTemp = dec5 wSeqStart\5
		gosub TASerOut[cSSC_OUT, wSSC_BAUD, @szSSCEEPromTemp, 5]		; output command to the SSC
		szSSCEEPromTemp(0) = ","
		for i = 0 to 31
			szSSCEEPromTemp(1)=dec3 ab(wBufStart)\3
			gosub TASerOut[cSSC_OUT, wSSC_BAUD, @szSSCEEPromTemp, 4]		; output next byte value
			wBufStart = wBufStart + 1
		next
		gosub TASerOut[cSSC_OUT, wSSC_BAUD, @CRSTR, 1]		; output the CR
		cbLeft = cbLeft - 32
		wSeqStart = wSeqStart + 32
		toggle p6
	pause 10
	wend
	if cbLeft then
		gosub TASerOut[cSSC_OUT, wSSC_BAUD, @EEWSTR, 5]		; output command to the SSC
		szSSCEEPromTemp = dec5 wSeqStart\5
		gosub TASerOut[cSSC_OUT, wSSC_BAUD, @szSSCEEPromTemp, 5]		; output command to the SSC
		szSSCEEPromTemp(0) = ","
		for i = 0 to cbLeft-1
			szSSCEEPromTemp(1)=dec3 ab(wBufStart)\3
			gosub TASerOut[cSSC_OUT, wSSC_BAUD, @szSSCEEPromTemp, 4]		; output next byte value
			wBufStart = wBufStart + 1
		next
		gosub TASerOut[cSSC_OUT, wSSC_BAUD, @CRSTR, 1]		; output the CR
		toggle p6
		pause 10
	endif
	
	return

;------------------------------------------------------------------------------
; DumpEEProm: - Read SSC BAP version
;------------------------------------------------------------------------------
DumpEEProm[wDumpStart, wDumpCnt]:
	serout s_out, i38400, ["Start Dump: ", hex wDumpStart, " Count: ", dec wDumpCnt, 13]
	while wDumpCnt > 0
		gosub ReadSSCEEPPromMem[wDumpStart, wDumpCnt max 16,   @ab]
		
		serout s_out, i38400, [hex4 wDumpStart\4,":"]
		
		; first dump the characters out in hex
		for iDump = 0 to 15
			if _TM_bDumpMode = 0 then
				if iDump < wDumpCnt then
					serout s_out, i38400, [hex2 ab(iDump)\2, " "]
				else
					serout s_out, i38400, ["   "]
				endif
			elseif ((iDump & 0x1) = 0)	; sort-of crap - but word size so only output every other byte
				if iDump < wDumpCnt then
					if _TM_bDumpMode = 1 then
						wB.highbyte = ab(iDump)
						wB.lowbyte = ab(iDump+1)
						serout s_out, i38400, [dec5 wB\5, " "]
					else
						sB.highbyte = ab(iDump)
						sB.lowbyte = ab(iDump+1)
						if sB < 0 then
							serout s_out, i38400, [sdec5 sB\5, " "]
						else
							serout s_out, i38400, [sdec6 sB\6, " "]
						endif
					endif
				else
					serout s_out, i38400, ["       "]
				endif
			endif
		next
		
		; then dump them out in ascii
		for iDump = 0 to ((wDumpCnt -1) max 15)
			if (ab(iDump) > 0x1f) and (ab(iDump) < 0x7f) then
				serout s_out, i38400, [ab(iDump)]
			else
				serout s_out, i38400, ["."]
			endif
		next
		
		serout s_out, i38400, [13]	; output the cr to end the line
		; setup for the next line if any
		if wDumpCnt > 16 then
			wDumpCnt = wDumpCnt - 16
			wDumpStart = wDumpStart + 16
		else
			wDumpCnt = 0
		endif	
	wend	
	
	return	
	
#endif

#ifdef TMA
; for now only BAP28...
;==============================================================================
; Subroutine: FindSSCBaudRate - This function will find what the baud rate is by
;			trying to issue a VER command to the SSC. If this fails it will
;			try a different baud rate. Should not be hard as there are only 4,
;
;==============================================================================
VERSTR		bytetable "ver",13
EMPTYSTR	bytetable	13, 13, 13

FindSSCBaudRate:
	cbSeq = 0		; reused variable...
	wSSC_BAUD = i115200
	while cbSeq = 0
;		Serout s_out, i9600, ["Try Ver at :", dec wSSC_BAUD & 0x1fff, 13]
		gosub TASerOut[cSSC_OUT, wSSC_BAUD, @VERSTR, 4]		; output command to the SSC
		gosub TASerIn[cSSC_IN, 0, wSSC_BAUD, @ab, 30, 25000, 13], cbSeq
		
		if cbSeq then
;			serout s_out, i9600, ["SSC Baud: ", dec wSSC_BAUD & 0x1fff, " Version: ", str szSSCEEPromTemp\30\13, 13]
		else
			if wSSC_BAUD = i115200 then
				wSSC_BAUD = i38400
			elseif wSSC_BAUD = i38400
				wSSC_BAUD = i9600
			elseif wSSC_BAUD = i9600
				wSSC_BAUD = i2400
			else
;				serout s_out, i9600, ["!!! Could not retrieve SSC Version !!!", 13]
				wSSC_BAUD = i38400
				cbSeq = 1
			endif
			pause 250
			gosub TASerOut[cSSC_OUT, wSSC_BAUD, @EMPTYSTR, 3]	; try to clear out buffer...
			pause 250
			
		endif	
	wend
	return



; Using my Serial output functions as I may be running at full speed...

;==================================================================================================
;==================================================================================================
; Timer Based Serial Input and Output Functions.
;==================================================================================================
;==================================================================================================
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;==============================================================================
; [TASerout(pin, baudmode, buffer, bufferlen]
;
; Second attempt will be to use TimerV which is only 8 bits, this version will
; use the Compare Match A interrupt do do the acutual output bit processing.
; The main loop will still wait in for the work to be done.  Could later change
; this to queue the values and do it all in the background, but that could get
; complicated as maybe I would start doing multiple different IO ports at the same
; time...  But maybe later.
;
;==============================================================================
; define parameters to this function.
TASPin 		var	byte
TASFPIN		var	byte
TASBM		var	word
TASPBUF 	var	POINTER
TASCnt 		var	word
TATimeout	var	long
TAbEOL		var	word

; TASerial definitions
_TASI_cFO	var	byte			; how many bytes have we cached away in our serial buffer

TASerout[TASPin, TASBM, TASPBUF, TASCNT]
	; do a simple call to our assembly language function
	jsr _TASEROUT_SUB:8
	return

;==============================================================================
; [TASerin(pin, fPin, baudmode, buffer, bufferlen, timeout, wEOL]
;	This is a timer based input function.  This one will also try to handle
;	some flow control and handle up to two extra bytes of data being sent
;	after we toggle the flow control pin as for example the XBEE may send
;	one or two characters at that point...
;   Also it can optionally exit when an optional end character <> 0 is read
;
; BUGBUG: still some timing issues setting up the proper timer tick as we need
;		to wait until we see the edge of the input come in and then we need
;		to guess where the center of the start bit should be...
;
; BUGBUG: When flow control is used, we try to cache up bytes that may be lost
;		after the flow control pin goes back high/low and use it at the start
;	    of the next read.  We don't verify that the actual read is for the same
;		IO pin, however it may be ok as we only look at and muck with this data
;		if a flow control pin is specified on the read.
;
;
;==============================================================================
_TASI_FOBuf	var	byte(4)			; used for flow control overrun.

TASerin[TASPin, TASFPin, TASBM, TASPBUF, TASCNT, TATIMEOUT, TABEOL]:
	; do a simple call to our assembly language function
	jsr _TASERIN_SUB:8
	return TASCNT		; return count actually read...

;==============================================================================
; [TASClearFOBuf - clear any input bytes that we cached.
;==============================================================================

TASClearFOBuf:
	_TASI_cFO = 0		; simply zero the count
	return 


;==============================================================================
BEGINASMSUB
ASM{
.macro	WAIT_TIMER_COMPARE_MATCH
	bld.b	#6, @TCSRV:8			; (4 byte instruction)see if we had a timer cycle go through
	bcc		#.-4:8					; nope loop back
	bclr.b	#6, @TCSRV:8			; clear out the status
.endm

;==============================================================================
; _TAS_INITTIMER - Internal function to do the initialize the timer based on
;			input baud mode.
;==============================================================================
_TAS_INITTIMER:
	; ok need to initialize TimerV
	mov.w	@TASBM:16, r1			; get the baudmode.
	
	bld.b	#6, r1h					; Get the normal/invert bit into C
	subx.b	r3h, r3h				; r3h = 0 if Normal or ff if inverted
	and.w	#0x1fff, r1				; extract the bit time
.ifdef BAP40						; if we are running on a bap40 we need to modify the baud rate by multply by 1.25
	mov.w	r1,e1					; save *1
	shll.w	r1						; * 2
	shll.w	r1						; * 4
	add.w	e1, r1					; *4 + *1 = *5
	shlr.w	r1						; *5/2 We will start out the timer/4 instead of 8 so don't need to divide last tim
;	shlr.w	r1						; *5/4 ; BUGBUG:: if r1h=0 we could start at /2
	mov.w	#2,e1					; Start off dividing clock by 4 as we multiplied the value by 2 by our multiply by 5/2
.else
	mov.w	#3,e1					; start off assuming clock will be divided by 8 as the baud mode is set this way
.endif	
_TASO_MCL:	
	or.b	r1h, r1h				; see if our count is < 256
	beq		_TASO_BMLD:8				; yep done with this loop
	shlr.w	r1						; nope divide by 2
	add.w	#1, e1					; change clock input for TimerV
	bra		_TASO_MCL:8				; Check again

_TASO_BMLD:	
	; BUGBUG: should verify that we are in a valid range...
	; bugbug;;; lets try decrementing by 1...
	dec.b	r1l
	mov.b	r1l, @TCORA:8			; save away the value that we should count up to for each bit
.ifdef DEBUG_ASM
	mov.b	r1l, @XXXTCORA:16		; DEBUG...
.endif	
	mov.w	e1, r1
	mov.b	#0, r1h					; zero out the high byte
	rotr.w	r1						; so low order bit now in high bit of high word
	rotl.b	r1h						; now in low bit of r1h
	mov.b	r1h, @TCRV1:8			; OK so should have the low bit of timer calc stored properly away.
	or.b	#0x08, r1l				; Added in the bits: CCLR0 for the TCRV0
	mov.b	r1l, @TCRV0:8			; save away the generated configuration byte
.ifdef DEBUG_ASM
	mov.b	r1l, @XXXTCRV0:16		; DEBUG...
	mov.b	r1h, @XXXTCRV1:16		; DEBUG...
.endif		
	rts

;==============================================================================
; _TASEROUT_SUB - Assembly language subroutine to do a serial output using timerV
;                 to time the different BITS.
;==============================================================================
_TASEROUT_SUB:
	jsr		_TAS_INITTIMER:16		; use shared function to initialize the timer
	
; Now lets use the Porttable that BAP has to convert logical pin to Port/pin
	xor.l	er0, er0				; zero out er0
	mov.b	@TASPIN:16, r0l			; move the logical pin number into r0l
	and.b	#0x3f, r0l				; make sure we are in the range 0-63
	shll.b	r0l						; double the offset as the port table is 2 bytes per entry.
	mov.w	@(_PORTTABLE:16, er0), r2	; OK R2 now has the two bytes high being the bit, and low being the port...
	mov.b	r2h, r3l				; Ok r3l now has the pin number
	mov.b	#0xff, r2h				; setup 32 bit address to port register PDRx
	
	; make sure it is setup for output
	mov.b	@(0x70-0xD0, er2), r0l	; get the current value for PCRx byte from the shadow location
	bset.b	r3l, r0l				; set the appropriate bit for our port
	mov.b	r0l, @(0x70-0xD0, er2)	; save away the updated value to the shadow location
	mov.b	r0l, @(0x10, er2)		; save away in the actual PCRx byte
		
	; Make sure the IO line is correct to start off with
	or.b	r3h, r3h
	beq		_TASO_INORMAL:8			; OK will start up normal
	bset.b	r3l,@er2				; Inverted mode start high
	bra		_TASO_INITBUF:8			; continue to setup pointers to buffers
_TASO_INORMAL:
	bclr.b	r3l, @er2				; this sets the IO line to low...

	; Now setup data pointer and counter.
_TASO_INITBUF:
	mov.l	@TASPBUF:16, er4		; OK now ER4 has pointer to the buffer to output.
	mov.w	@TASCNT:16, e3			; Now E3 has the count of bytes to output
	WAIT_TIMER_COMPARE_MATCH		; wait for timer match

	;------------------------------------------------
	; Main loop to output our data.  The IO line should
	; be initialized and should be in the stop state...
	;
	; Current registers are:
	; r2	- Word address for PDRx for the output pin
	; r3l	- bit number of IO PIN
	; r3h	- 0 if Normal communication, FF if inverted.
	; e3	- Count of bytes left to output	
	; er4 	- Pointing to the current byte to output

	; Wait at least one timer unit before we start
	WAIT_TIMER_COMPARE_MATCH			; wait for timer match

_TASO_OUTPUT_NEXT_BYTE:
	mov.b	@er4+, r0l					; get the next byte that we will output
	xor.b	r3h, r0l					; take care of the inversion of bits
	mov.b	#8, r0h						; number of bits to output.
	; OK now lets setup and do that start bit...
	; put up to one bit delay at the start to get everything in sync...
	WAIT_TIMER_COMPARE_MATCH			; wait for the next clock.

	or.b	r3h,r3h							; (2)see if we are in inverted mode
	bne		_TASO_STARTINVERT:8				; (4)
	; normal inverted.
	bset.b	r3l,@er2						; (=6) Mormal mode Start goes high
	bra		_TASO_BITLOOP:8
_TASO_STARTINVERT:
	bclr.b	r3l, @er2						; (=6) this sets the IO line to low...
	
_TASO_BITLOOP:
	WAIT_TIMER_COMPARE_MATCH				; wait for the next timer

	shlr.b	r0l								; (2) get the next data bit to output
	bcc		_TASO_BIT0:8					; (4) not set so was zero
	bclr.b	r3l,@er2						; (=6)The bit is a zero
	bra		_TASO_BITCLEANUP:8				; 

_TASO_BIT0:
	bset.b	r3l, @er2						;(=6) Ok this bit goes high

_TASO_BITCLEANUP:
	dec.b	r0h								; decrement our count of bits to process
	bne		_TASO_BITLOOP:8					; still more bits to go.
	
	
	; now lets do the stop bit
	WAIT_TIMER_COMPARE_MATCH				; wait for the next timer
	or.b	r3h, r3h						; (2)
	bne		_TASO_STOPINVERT:8				; (4)

	; normal inverted.
	bclr.b	r3l, @er2						; (=18) this sets the IO line to low...
	bra		_TASO_CHECK_FOR_MORE_BYTES:8		

_TASO_STOPINVERT:
	bset.b	r3l,@er2						; (=18) Mormal mode Start goes high

_TASO_CHECK_FOR_MORE_BYTES:			
	dec.w	#1, e3							; decrement the count of bytes we have left to do
	bne		_TASO_OUTPUT_NEXT_BYTE:8		; still more to go.	

	; and finally return 
	rts

;==============================================================================
; _TASERIN_SUB - Assembly language subroutine to do a serial input using timerV
;                 to time the different BITS.
;==============================================================================
_TASERIN_SUB:
	push.l	er6						; May use ER6 - must save and restore
	xor.l	er6, er6				; clear it for the normal case
	jsr		_TAS_INITTIMER:16		; use shared function to initialize the timer

	; Now setup data pointer and counter.
	mov.l	@TASPBUF:16, er4		; OK now ER4 has pointer to the buffer to read into
	mov.w	@TASCNT:16, e3			; Now E3 has the count of bytes to input

	xor.l	er0, er0				; zero out er0

	; Now check to see if we have a flow control pin to muck with.
	mov.b	#0xff, r1l				; if no flow control this will be -1
	mov.b	@TASFPIN:16, r0l		; get the flow control pin
	beq		_TASI_INIT_NO_FLOW_PIN:8; no flow control pin

	; Also lets setup our default timeout for reading in overflow
	mov.w	@TASBM:16, e1			; get the baudmode.
	and.w	#0x1fff, e1				; extract the bit time
	shll.w	e1	

	; BUGBUG 0 is a valid pin, maybe should check for -1 or the like?
	; If we are doing flow control, then check to see if we have run over bytes
	; that were sent after our last serin was done.  If so use those now for the start
	; of this serin...
	mov.b	@_TASI_CFO, r1l			; count of bytes in our overflow buffer
	beq		_TASI_END_PROCESS_OVERRUN:8	; nothing to use

	; BUGBUG: we have not verified that the overrun is for our pin and baud mode...	
	mov.l	#_TASI_FOBUF:32, er2	; pointer to our overflow buffer

	mov.w	@TABEOL:16, r5			; Got the char = could save away for later...

	;-----------------------------------------------------------
	; At this loop we have the following registers set or used
	; r0l	- Flow control pin number
	; r1l	- Count of bytes in the overflow buffer
	; r1h 	- used to transfer bytes
	; e1	- Baud Mode
	; er2	- pointer to overflow buffer
	; e3	- Count of bytes left to read
	; er4 	- Pointing to the current byte to output
	; r5	- Use for optional EOL character...
	
_TASI_PROCESS_OVERRUN_LOOP:
	mov.b	@er2+, r1h				; get the next byte post increment
	mov.b	r1h, @er4				; save it away
	inc.l	#1, er4					; point to the next byte
	dec.w	#1,e3					; decrement how many bytes left to be read
	beq		_TASI_FLOW_EXIT_DONE:16	; overflow filled us.
	or.b	r5h, r5h				; do we have an EOL character
	bne		_TASI_OVERRUN_NO_EOLCHAR:8
	cmp.b	r1h, r5l				; does it match our EOL character?
	beq		_TASI_FLOW_EXIT_DONE:16	; overflow filled us.
_TASI_OVERRUN_NO_EOLCHAR:
	dec.b	r1l						; decrement our count of overflow bytest left to process
	bne		_TASI_PROCESS_OVERRUN_LOOP:8	; process the next one...

_TASI_END_PROCESS_OVERRUN:
	and.b	#0x3f, r0l				; make sure we are in the range 0-63
	shll.b	r0l						; double the offset as the port table is 2 bytes per entry.
	mov.w	@(_PORTTABLE:16, er0), r6	; OK R6 now has the two bytes high being the bit, and low being the port...
	mov.b	r6h, r1l				; Ok r1l now has the pin number
	mov.b	#0xff, r6h				; setup 32 bit address to port register PDRx
	
	; make sure it is setup for output
	mov.b	@(0x70-0xD0, er6), r0l	; get the current value for PCRx byte from the shadow location
	bset.b	r1l, r0l				; Set the appropriate bit for our port
	mov.b	r0l, @(0x70-0xD0, er6)	; save away the updated value to the shadow location
	mov.b	r0l, @(0x10, er6)		; save away in the actual PCRx byte
	
	; And now set the flow control line depending on the invert bit
	or.b	r3h, r3h				; check for inverted
	bne		_TASI_FLOW_INIT_INVERT:8
	bset.b	r1l,@er6				; Not inverted - so set the flow control line high
	bra		_TASI_FLOW_INIT_DONE:8	; 
_TASI_FLOW_INIT_INVERT:
	bclr.b	r1l,@er6				; Inverted set the flow control low...

; Now lets use the Porttable that BAP has to convert logical pin to Port/pin
_TASI_FLOW_INIT_DONE:	
	mov.b	@_TASI_CFO, r5l			; count of bytes in our overflow buffer
	extu.w	r5
	mov.w	r5, e6					; Needed to initialize E6 to the number of bytes we actually transfered so far...

_TASI_INIT_NO_FLOW_PIN:
	mov.b	@TASPIN:16, r0l			; move the logical pin number into r0l
	and.b	#0x3f, r0l				; make sure we are in the range 0-63
	shll.b	r0l						; double the offset as the port table is 2 bytes per entry.
	mov.w	@(_PORTTABLE:16, er0), r2	; OK R2 now has the two bytes high being the bit, and low being the port...
	mov.b	r2h, r3l				; Ok r3l now has the pin number
	mov.b	#0xff, r2h				; setup 32 bit address to port register PDRx
	
	; make sure it is setup for input
	mov.b	@(0x70-0xD0, er2), r0l	; get the current value for PCRx byte from the shadow location
	bclr.b	r3l, r0l				; Clear the appropriate bit for our port
	mov.b	r0l, @(0x70-0xD0, er2)	; save away the updated value to the shadow location
	mov.b	r0l, @(0x10, er2)		; save away in the actual PCRx byte

	; Get the optional EOL character into E0.  We use 16 bits as that allows us to not have
	; to reserve any characters that we can not test for If high word is non zero we will
	; not match anything.
	mov.w	@TABEOL:16, e0			; Got the char

	; When we get the first edge of the start bit, we want to adjust the Timers Count
	; such that we should get overflow near the middle of the start pulse.  Preferably
	; a slight be earlier then center
	; When we have the start we want to setup our timer to be half way through
	; a bit cycle to read in the center of it.
	mov.b	@TCORA:8, r0l			; Get the pulse width 
	mov.b	r0l, r1h				; lets try setting at the 3/4 location in pulse
	shlr.b	r0l						; divide in half
	shlr.b	r0l						; 1/4 
	sub.b	r0l, r1h				; R1h is approx 3/4 of the value
	
;	xor.w	e6, e6					; Not Needed if no flow pin then still zero and
									; if flow pin init to what was in overflow buffer
	;=============================================================================
	; OK we now have init everything, call of to process the input data
	; 
	; Registers:
	; r0l	- The character being input
	; r0h	- count of bits left for the current byte
	; e0	- Optional EOL character such as CR - high byte <>0 is no check...
	; r1h	- Timer start adjust after leading edge of start pulse
	; r1l	- bit number of Flow IO pin. -1 if no flow control
	; e1	- Default timeout for overflow processing
	; r2	- Word address for PDRx for the input pin
	; r3l	- bit number of IO PIN
	; r3h	- 0 if Normal communication, FF if inverted.
	; e3	- Count of bytes left to output	
	; er4 	- Pointing to the current byte to output
	; er5	- We will use for timeout processing...
	; e6	- Use for byte counter (need to make sure ok for flow pin later...
	; er6 	- word address of PDRx for flow control pin or 0
	;
	; External variables that are also used in the funciton.
	; 	TATIMEOUT - timer timeout
_TASI_INPUT_NEXT_BYTE:
	; First we need to wait for the start bit to start
	; BUGBUG:: Put in time out here...
	mov.l	@TATIMEOUT:16, er5			; Get our timeout value

	or.b	r3h, r3h					; our we in inverted mode?
	bne		_TASI_WAIT_START_INVERT:8	; yes
	
_TASI_WAIT_START_NON_INVERT:			;
	btst.b	r3l,@er2					;(6)get the state
	bne		_TASI_SET_WE_HAVE_START_BIT:8	;(4) Ok we have our start
.ifdef BAP40		
	nop									; if on a 40 add 4 clocks for timeing
	nop
.endif		
	dec		#1, er5							;(2)
	bne		_TASI_WAIT_START_NON_INVERT:8	;(4) not in our state yet - 1 loop per uS
	bra		_TASI_TIMEOUT:16			; We had a timeout
	
_TASI_WAIT_START_INVERT:				;
	btst.b	r3l,@er2					; get the state not bit -> Z
	beq		_TASI_SET_WE_HAVE_START_BIT:8	; Ok we have our start
.ifdef BAP40		
	nop									; if on a 40 add 4 clocks for timeing
	nop
.endif		
	dec		#1, er5							;(2)
	bne		_TASI_WAIT_START_INVERT:8	;(4) not in our state yet - 1 loop per uS
	bra		_TASI_TIMEOUT:16			; We had a timeout

		
_TASI_SET_WE_HAVE_START_BIT:			; Ok we have our start

	; When we have the start we want to setup our timer to be half way through
	; a bit cycle to read in the center of it.
	mov.b	r1h, @TCNTV:8				; Was calculated before call.
	bclr.b	#6, @TCSRV:8				; clear out the overflow
 	
	xor.b	r0l,r0l						; start off with a zero value... May not be necessary
	mov.b	#8, r0h						; number of bits to output.

	; First timer should now be in start of start bit
	WAIT_TIMER_COMPARE_MATCH				; wait for the next timer
.ifdef DEBUG_WITH_LEDS
	bnot.b	#4, @PDR5:8				; BUGBUG::: say we got here.
.endif
	; Now for the main bit loop.
	
_TASI_BITLOOP:
	WAIT_TIMER_COMPARE_MATCH				; wait for the next timer
.ifdef DEBUG_WITH_LEDS
	bnot.b	#5, @PDR5:8				; BUGBUG::: say we got here.
.endif
	shlr.b	r0l								; shift it down
	btst.b	r3l,@er2						; get the state not bit -> Z
	bne		_TASI_BIT0						; the bit was zero
	bset.b	#7, r0l							; was a 1 so or it in...

_TASI_BIT0:
	dec.b	r0h								; decrement our count of bits to process
	bne		_TASI_BITLOOP:8					; still more bits to go.

	; While waiting for the stop bit, lets use some of the time to save away the character...
	xor.b	r3h, r0l						; take care of the inversion of bits
	mov.b	r0l, @er4						; Save away the byte that we received
	inc.l	#1, er4							; increment pointer to next place to store char	
	inc.w	#1, e6							; increment the number of bytes we saved counter.
	
	; try doing this before STOP bit arrives to save the time after...
	cmp.w	e0, r0							; 2 see if the EOL matches - R0H should be zero (no bits left so word compare should work)
	bne		_TASI_WAIT_FOR_STOP_BIT:8		; 4 Did not match so process next character
	
	; EOL matched
	mov.w	#1, e3							; hack so the decrement after the STOP will process this as well...
	; now lets do the stop bit - BUGBUG:: should probably validate that it is a proper stop bit, but...
_TASI_WAIT_FOR_STOP_BIT:
	WAIT_TIMER_COMPARE_MATCH				; wait for the next timer
.ifdef DEBUG_WITH_LEDS
	bnot.b	#5, @PDR5:8				; BUGBUG::: say we got here.
.endif
	; Now see if we have more characters to process - Also check to see if we have an end character to match...
	dec.w	#1, e3							; 2 decrement the count of bytes we have left to do
	bne		_TASI_INPUT_NEXT_BYTE:8			; 4 Did not match so process next character

	; matched EOL or count went down to zero so fall through

_TASI_TIMEOUT:
	cmp.w 	#0xffff,r6						; 4 See if we have a flow control IO port
	beq	 	_TASI_FLOW_END:8				; 4 We have flow control and this is the second time through

	; first time though save away the updated count.
	mov.w	e6, @TASCNT:16					; 6 which should give us count of bytes we received.

	; If not flow control bail
	or.w 	r6,r6							; 2 See if we have a flow control IO port
	beq	 	_TASI_NOFLOW_END:16				; 4 no flow control...
	
	xor.w	e6, e6							; 2 zero out again to make sure this works and for counter for second pass
	bnot.b	r1l,@er6						; 8 Simply invert the line - it was initialized earlier
	mov.w	#0xffff, r6						; 4 Set to special state to know we were here before
	
	mov.l	#_TASI_FOBUF:32, er4			; 6 Ok lets setup to use our internal buffer
	mov.w	#4, e3							; 4 max number of bytes to buffer up
	mov.w	#0xffff, e0						; 4 Don't look for EOL characters here
	
	mov.w	e1, r5							; 2
	extu	er5								; 2 so er1 should be about 4 bits in width...
	mov.l	er5, @TATIMEOUT:16				; 10 update the timeout variable...

	or.b	r3h, r3h						; 2 our we in inverted mode?
	bne		_TASI_WAIT_START_INVERT:16		; 6 yes
	bra		_TASI_WAIT_START_NON_INVERT:16	; 6 	
	; could have done simple branch to start, but save a few cycles just in case...

_TASI_FLOW_EXIT_DONE:
	;-----------------------------------------------------------
	; The flow buffer filled us up or contained our EOL character,
	; we did not need to ask for more data. 
	; so did not toggle flow control at all.
	; At this loop we have the following registers set or used
	; r1l	- Count of bytes in the overflow buffer
	; r1h 	- used to transfer bytes
	; er2	- pointer to overflow buffer
	; er4 	- Pointing to the current byte to output
	; so first save away how many bytes we transfered.	
	mov.l	@TASPBUF:16, er5				; get the start pointer
	sub.l	er5, er4						; subtract start of buffer from where we are now
	mov.w	r4, @TASCNT:16					; which should give us count of bytes we received.

	; now see if we have more bytes left in our buffer.  We did not decrement for the last byte
	; we did transfer so do it first
	dec.b	r1l								; decrement our count of overflow bytest left in the buffer
	mov.b	r1l, @_TASI_CFO:16				; save away how many bytes we have left...
	beq		_TASI_NOFLOW_END:8				; no bytes left to output.
	
	mov.l	#_TASI_FOBUF:32, er4			; pointer to our overflow buffer to transfer stuff up to start of...
_TASI_FLOW_EXIT_TRANSER_UNUSED_TO_FRONT:
	mov.b	@er2+, r0l						; get byte to transfer to start
	mov.b	r0l, @er4						; Save at start
	inc.l	#1, er4							; point to next char
	dec.b	r1l
	bne		_TASI_FLOW_EXIT_TRANSER_UNUSED_TO_FRONT:8	; more to go...
	
	bra		_TASI_NOFLOW_END:8				; then return 	
	
_TASI_FLOW_END:	
	; e6 now has the count of bytes we received so simply use it.
	mov.w	e6, r4							; can not move .b out of e register
	mov.b	r4l, @_TASI_CFO:16				; which should give us count of bytes we received.
	; fall through to return

_TASI_NOFLOW_END:
	pop.l	er6								; This one has to be preserved...	
	; and finally return 
	rts
}
ENDASMSUB
#endif